var PopupSoundExtension = {
  player: null,
  clickSound: null,
  popupSound: null,
  downloadSound: null,
  throttle: 0,

  startup: function() {
     //listen to page navigations
    gBrowser.addProgressListener(PopupSoundExtension.navigationListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);

     //listen to popup blocker
    gBrowser.addEventListener("DOMPopupBlocked", PopupSoundExtension.onPopupBlocked, false);

     //listen to download manager
    Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService)
        .addObserver(PopupSoundExtension.downloadObserver, "dl-done", false);

    PopupSoundExtension.loadSounds();
  },

  destroy: function() {
    window.removeEventListener("load", PopupSoundExtension.startup, false);
    gBrowser.removeProgressListener(PopupSoundExtension.navigationListener);
    gBrowser.removeEventListener("DOMPopupBlocked", PopupSoundExtension.onPopupBlocked, false);
    Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService)
        .removeObserver(PopupSoundExtension.downloadObserver, "dl-done", false);
    PopupSoundExtension.unloadSounds();
  },

  loadSounds: function() {
    const prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    const ios = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces.nsIIOService);

    try {
      if(prefs.getBoolPref("extensions.popupsound.firstlaunch"))throw "";
    } catch(e) {
      prefs.setBoolPref("extensions.popupsound.firstlaunch", false);
      window.openDialog("chrome://popupsound/content/config.xul","_blank", "chrome,resizable=yes");
    }

    PopupSoundExtension.player = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    PopupSoundExtension.player.init();

    const _getURL = function(p) {
      try {
        var file = prefs.getComplexValue(p, Components.interfaces.nsILocalFile);
        return ios.newFileURI(file);
      } catch(e) {
        try {
          var str = prefs.getComplexValue(p, Components.interfaces.nsISupportsString).data;
          if(str == "")return null;
          return ios.newURI(str, null, null);
        } catch(e2) {
          dump("\nPopupSound Extension:"+e2);
        }
      }
    };

    PopupSoundExtension.popupSound = _getURL("privacy.popups.sound_url");
    PopupSoundExtension.clickSound = _getURL("browser.link.click_sound_url");
    PopupSoundExtension.downloadSound = _getURL("browser.download.manager.finished_sound_url");
  },

  unloadSounds: function() {
    PopupSoundExtension.player = null;
    PopupSoundExtension.popupSound = null;
    PopupSoundExtension.clickSound = null;
    PopupSoundExtension.downloadSound = null;
  },

  onPopupBlocked: function(event) {
    if(PopupSoundExtension.popupSound && Date.now() - PopupSoundExtension.throttle > 100) {
      PopupSoundExtension.throttle = Date.now();
      PopupSoundExtension.player.play(PopupSoundExtension.popupSound);
    }
  },

  onStartNavigate: function() {
    if(PopupSoundExtension.clickSound && Date.now() - PopupSoundExtension.throttle > 100) {
      PopupSoundExtension.throttle = Date.now();
      PopupSoundExtension.player.play(PopupSoundExtension.clickSound);
    }
  },

  onDownloadComplete: function() {
    if(PopupSoundExtension.downloadSound && Date.now() - PopupSoundExtension.throttle > 100) {
      PopupSoundExtension.throttle = Date.now();
      PopupSoundExtension.player.play(PopupSoundExtension.downloadSound);
    }
  },

  navigationListener: {
    START: Components.interfaces.nsIWebProgressListener.STATE_START,
    onStateChange: function(a, b, flags, d) {
      if(flags & PopupSoundExtension.navigationListener.START) {
        PopupSoundExtension.onStartNavigate();
      }
    },
    onLocationChange: function(a, b, c) {},
    onProgressChange: function(a, b, c, d, e, f) {},
    onSecurityChange: function(a, b, c) {},
    onStatusChange: function(a, b, c, d) {},
    onLinkIconAvailable: function(a, b) {},
    QueryInterface: function(id) {
      if(id.equals(Components.interfaces.nsIWebProgressListener) ||
         id.equals(Components.interfaces.nsISupportsWeakReference) ||
         id.equals(Components.interfaces.nsISupports))
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
  },

  downloadObserver: {
    observe: function(a, topic, c) {
      if(topic == "dl-done") {
        PopupSoundExtension.onDownloadComplete();
      }
    }
  }
};

window.addEventListener("load", PopupSoundExtension.startup, false);
window.addEventListener("unload", PopupSoundExtension.destroy, false);